Option Strict On           'Not necessary but improves compatibility
'Imports the Marshal class
Imports System.Runtime.InteropServices

Public Class ReadaComm
    'Declarations
    Private Declare Auto Function FindWindowW Lib "user32.dll" (ByVal lpClassName As String, ByVal lpWindowName As String) As Int32
    Private Declare Auto Function RegisterWindowMessage Lib "user32.dll" (ByVal lpString As String) As Integer
    Private Declare Auto Function SendMessage Lib "user32.dll" (ByVal hWnd As IntPtr, ByVal msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As IntPtr

    'Structure used when passing data with wm_CopyData
    Private Structure CopyData
        Public dwData As IntPtr
        Public cbData As Integer
        Public lpData As IntPtr
    End Structure


    'The Register button sends True to the RegWithRAC function who sends a register message to Read-a-Card
    Private Sub RegBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RegBtn.Click
        RegisterWithRAC(True)
    End Sub


    'The Unregister button sends False to the RegWithRAC function who sends an unregister message to Read-a-Card
    Private Sub UnregBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UnregBtn.Click
        RegisterWithRAC(False)
    End Sub


    'Read-a-Card must be registered with in order to receive messages when a card is swiped.
    'This button creates a unique Windows message ID, finds the Read-a-Card window, and sends
    'it its own window handle telling Read-a-Card to register or unregister it
    Private Sub RegisterWithRAC(ByVal regStat As Boolean)
        'Holds the Read-a-Card window handle
        Dim targetWinHandle As IntPtr

        'Create a registered Windows message ID to avoid collision
        Dim msgID As Integer = RegisterWindowMessage("READACARD")

        'Find the Read-a-Card window and assign its handle as your target
        If (FindWindowW(Nothing, "Read-a-Card") <> 0) Then
            targetWinHandle = CType(FindWindowW(Nothing, "Read-a-Card"), IntPtr)
        ElseIf (FindWindowW(Nothing, "Read-a-Card (TRIAL MODE)") <> 0) Then
            targetWinHandle = CType(FindWindowW(Nothing, "Read-a-Card (TRIAL MODE)"), IntPtr)
        End If

        'Send your own window handle to the Read-a-Card window to register (WPARAM > 0) or unregister (WPARAM = 0)
        If (regStat) Then
            'The integer 3 will register with Read-a-Card and disable keyboard wedge functionality
            SendMessage(targetWinHandle, msgID, CType(3, IntPtr), Me.Handle)
        Else
            SendMessage(targetWinHandle, msgID, CType(0, IntPtr), Me.Handle)
        End If
    End Sub


    'The Windows value for wm_CopyData type messages is &H4A, used below
    Private Const WM_COPYDATA As Double = &H4A

    'Override the default WndProc to handle WM_COPYDATA messages
    Protected Overrides Sub WndProc(ByRef recWinMessage As System.Windows.Forms.Message)

        'Only do this when the received winMessage is of type wm_CopyData
        If recWinMessage.Msg() = WM_COPYDATA Then
            'Holds our received copy of winMessage.lParam
            Dim Data As CopyData

            'Used to step through the received CopyData.lpData
            Dim ptr As IntPtr

            'Character arrays to hold our Read-a-Card data
            Dim cardID(256) As Char
            Dim readerSerial(256) As Char
            Dim cardType(256) As Char
            Dim timestamp(256) As Char

            'copy the CopyData structure in the recWinMessage to our own CopyData structure
            Data = CType(recWinMessage.GetLParam(GetType(CopyData)), CopyData)

            'Data.lpData holds the pointer to the data strucutre from Read-a-Card
            'Unpack the structure into our character arrays
            ptr = Data.lpData
            Marshal.Copy(ptr, cardID, 0, 256)
            ptr = CType((ptr.ToInt64() + 512), IntPtr)
            Marshal.Copy(ptr, readerSerial, 0, 256)
            ptr = CType((ptr.ToInt64() + 512), IntPtr)
            Marshal.Copy(ptr, cardType, 0, 256)
            ptr = CType((ptr.ToInt64() + 512), IntPtr)
            Marshal.Copy(ptr, timestamp, 0, 256)

            'Sample output of unpacked winMessage containing our Read-a-Card data
            TextBox1.AppendText("Card ID = ")
            TextBox1.AppendText(CStr(cardID))
            TextBox1.AppendText(vbCrLf & "Reader ID = ")
            TextBox1.AppendText(CStr(readerSerial))
            TextBox1.AppendText(vbCrLf & "Card Type = ")
            TextBox1.AppendText(CStr(cardType))
            TextBox1.AppendText(vbCrLf & "Time Stamp = ")
            TextBox1.AppendText(CStr(timestamp))
            TextBox1.AppendText(vbCrLf & vbCrLf)

        End If

        'Call the BaseClass handler to ensure the "basics" are done
        MyBase.WndProc(recWinMessage)
    End Sub


    'Empty text box only to hold our sample output of the Read-a-Card data
    Private Sub TextBox1_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBox1.TextChanged

    End Sub

    'Label for text box
    Private Sub Label1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Label1.Click

    End Sub
End Class
